				/* Converte un numero k intero compreso
				   tra 0 e 36 in una cifra, col seguente criterio:
				    0  ->  '0'
				    1  ->  '1'
				    ..........
				    9  ->  '9'
				   10  ->  'a'
				   11  ->  'b'
				   ...........
				   34  ->  'y'
				   35  ->  'z'
				   Questo basta per rappresentare numeri in qualunque 
				   base tra 2 e 36. */
char bin_a_cifra(k)
     int k;
{
  if(k>=0 && k<10) return(k+'0');
  else if(k>=10 && k<=36) return(k-10+'a');
  else return('\000');
}
				/* Esegue la conversione da cifra a intero binario.
				   Il carattere ricevuto come argomento deve essere
				   una cifra decimale o un carattere alfabetico.
				   Accetta anche i caratteri maiuscoli.  
				   Se riceve un carattere non consentito restituisce -1.
				   */
int cifra_a_bin(c)
     char c;
{
  if(c>='0' && c<='9') return(c-'0');
  else if(c>='A' && c<='Z') return(c-'A'+10);
  else if(c>='a' && c<='z') return(c-'a'+10);
  else return(-1);
}
				/*  Esegue la conversione di un intero k da binario alla
				    rappresentazione in una base {\tt b} compresa tra 2 e 36.
				    La funzione esegue un controllo sulla correttezza del
				    parametro b, e restituisce il risultato come stringa di
				    caratteri che deve essere memorizzata in str.  
				    E' compito di chi chiama la funzione garantire che la
				    lunghezza della stringa str sia sufficiente a contenere la
				    rappresentazione, incluso il NUL che chiude la stringa.
				*/
void bin_a_base(k,b,str)
     int k;
     int b;
     char str[];
{
  int j,m;
  if(b<2 || b>36) {		/* Controlla la correttezza della base */
    printf("Errore in bin_a_base:\n");
    printf("La base b deve essere compresa tra 0 e 36. %d non e' valido.\n",b);
    str[0]='\000';
    exit(1);
  }
  if(k==0) {			/* Se k==0 restituisci la cifra '0' */
    str[0]='0'; str[1]='\000';
  }
  else {
    if(k<0) m= -k;  else m=k;	/* Converti k in numero positivo */
    for(j=0;m>0;j++) {		/* e calcola i resti delle divisioni successive per b */
      str[j]=bin_a_cifra(m%b);
      m/=b;
    }
    if(k<0) str[j++]='-';	/* Se k<0 aggiungi il segno '-' */
    str[j]='\000';		/* ed in ogni caso aggiungi il NUL finale */
    inverti_stringa(str);	/* Inverti la stringa per scriverla nell'ordine giusto. */
  }
  return;
}
				/*  Esegue la conversione di un intero k dalla rappresentazione
				    in una base {\tt b} compresa tra 2 e 36 a quella binaria.
				    La funzione esegue un controllo sulla correttezza del
				    parametro b, e restituisce il risultato come intero.
				    Inoltre la funzione controlla che la stringa contenga solo
				    caratteri che rappresentino effettivamente delle cifre in
				    base b.
				*/
int base_a_bin(b,str)
     int b;
     char str[];
{
  int j,m,k,neg;
  if(b<2 || b>36) {		/* Controlla la correttezza della base */
    printf("Errore in base_a_bin:\n");
    printf("La base b deve essere compresa tra 0 e 36. %d non e' valido.\n",b);
    str[0]='\000';
    exit(1);
  }
				/* Controlla se compare un segno; la variabile logica neg vale
				   "vero" se compare il segno '-', e "falso" se non c'e' segno
				   o compare un segno '+' */
  if((neg=(str[0]=='-')) || str[0]=='+') 
    j=1;			/* Se c'e' un segno salta il primo carattere della stringa */
  else j=0;			/* altrimenti il primo carattere di str e' una cifra  */
  for(m=0;str[j]!='\000';j++) {
    k=cifra_a_bin(str[j]);
    if(k<0 || k>=b) {
      printf("La stringa contiene il carattere %c, non consentito.\n",str[j]);
      exit(1);
    }
    m=m*b+k;
  }
  if(neg) m=-m;			/* se c'era un segno '-' cambia il segno di m; */
  return(m);			/* restituisci il risultato */
}
