/* parole.c 

   Trasforma un file di testo in un file contenente solo le parole del
    testo, separate dal carattere NUL.  Venono presi in considerazione
    i soli caratteri alfabetici.  
    Inoltre:
     o i caratteri minuscoli vengono trasformati nel corripsondente 
       maiuscolo;
     o i caratteri accentati in varii modi vengono trasformati nel 
       carattere maiuscolo piu' simile.
    La scelta di tradurre i caratteri accentati nel corrispondente 
    maiuscolo e' del tutto arbitraria, ovviamente, e si puo' modificare 
    a piacere.  Ad esempio, sarebbe ben corretto ritenere che un 
    carattere accentato e' ben diverso dal corrispondente non accentato, 
    e mantenere la differenza.  
    La corrispondenza tra caratteri viene creata da una tabella di 
    conversione, con nome tab. che viene inizializzata dal 
    sottoprogramma initab().  L'utilizzo e' semplice: al carattere 
    letto, che indico con c, viene sostituito tab[c].
    Il compattamento del testo viene ottenuto sopprimento tutti i NUL 
    che separano le singole parole, escluso uno.
    Per i caratteri che non hanno rappresentazione standard nel codice 
    ASCII si assume che venga usata la codifica ISO--8859-1 per 
    l'alfabeto latino.  Se il testo e' codificato in modo diverso oc

 */
#include <stdio.h>
#include <math.h>
#define NUL '\000'
				/* Dichiarazione ed inizializzazione
				   della tabella di conversione */
int tab[256] = { 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /*   0 -  15 */
		 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /*  16 -  31 */
		 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /*  32 -  47 */
		 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /*  48 -  63 */
		 0, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,	/*  64 -  79 */
		80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90,  0,  0,  0,  0,  0,	/*  80 -  95 */
		 0, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,	/*  96 - 111 */
		80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90,  0,  0,  0,  0,  0,	/* 112 - 127 */
		 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /* 128 - 143 */
		 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /* 144 - 159 */
		 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /* 160 - 175 */
		 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /* 176 - 191 */
		65, 65, 65, 65, 65, 65,  0, 67, 69, 69, 69, 69, 73, 73, 73, 73,	/* 192 - 207 */
		 0, 78, 79, 79, 79, 79, 79,  0,  0, 85, 85, 85, 85, 89,  0,  0,	/* 208 - 223 */
		65, 65, 65, 65, 65, 65,  0, 67, 69, 69, 69, 69, 73, 73, 73, 73,	/* 224 - 239 */
		79, 78, 79, 79, 79, 79, 79,  0,  0, 85, 95, 95, 95, 89,  0,  0	/* 240 - 255 */
};
				/* ### */
int main()
{
  FILE *in,*out;
  unsigned char c,cprec;
  char nomfil[256],nomout[256];
  long int ncarin,ncarou,nparol;
  int j=0;
				/* Apertura del file in lettura */
 vai:
  printf("Dimmi che file devo esaminare: ");
  scanf("%s",nomfil);
  if((in = fopen(nomfil,"r")) == NULL) {
      ++j;
      printf("Errore nell'apertura del file in lettura:\n");
      perror(nomfil);
      if(j<3) goto vai;
    else {
      printf("Hai sbagliato per tre volte. Vergognati!\n");
      return(-1);
    }
  }
				/* Apertura del file in scrittura */
  j=0;
 vai1:
  printf("Dimmi che file devo scrivere : ");
  scanf("%s",nomout);
  if((out = fopen(nomout,"w")) == NULL) {
    ++j;
    printf("Errore nell'apertura del file in lettura:\n");
    perror(nomout);
    if(j<3) goto vai1;
    else {
      printf("Hai sbagliato per tre volte. Vergognati!\n");
      return(-1);
    }
  }
				/* Procedo alla separazione delle parole */
  ncarin=ncarou=nparol=0;
  cprec=NUL;
  while(fread(&c,sizeof(char),1,in) != 0) {
    ++ncarin;
    if(tab[c]!=NUL) {
      cprec=tab[c];
      fwrite(&cprec,sizeof(char),1,out);
      ++ncarou;
    }
    else {
      if(cprec != NUL) {
	cprec=NUL;
	fwrite(&cprec,sizeof(char),1,out);
	++nparol;
      }
    }
  }
				/* Assicura che comunque l'ultimo carattere 
				   del file sia un NUL */
  if(cprec != NUL) {
    cprec=NUL;
    fwrite(&cprec,sizeof(char),1,out);
    ++nparol;
  }
  fclose(in);
  fclose(out);
  printf("Ho letto %ld caratteri;\n",ncarin);
  printf("Ho scritto %ld parole per un totale di %ld caratteri.\n",nparol,ncarou);
  return(0);
}


